@@ -1,4 +1,4 @@
-// $Id: ss_graph.cpp$
+// $Id: ss_graph.cpp,v 1.2 2004/11/24 12:07:46 ccb Exp $
 /*
      This library is free software; you can redistribute it and/or
      modify it under the terms of the GNU Lesser General Public
@@ -567,6 +567,21 @@ int i,j;
 
 }
 
+Boolean CSSGraph::isBuild()  {
+  if (!graph)  return False;
+  if (!Edge)   return False;
+  return True;
+}
+
+
+void  CSSGraph::calcVTypes() {
+int i;
+  nHelices = 0;
+  nStrands = 0;
+  for (i=0;i<nVertices;i++)
+    if (Vertex[i]->type==V_HELIX)  nHelices++;
+                             else  nStrands++;
+}
 
 void  CSSGraph::ReleaseEdges()  {
 int i;
@@ -1052,11 +1067,10 @@ void  CSSGraph::write ( RCFile f )  {
 int i;
 int Version=1;
 
-  f.WriteInt    ( &Version   );
-
-  f.CreateWrite ( name       );
-  f.WriteFile   ( devChain,sizeof(devChain) );
-  f.WriteInt    ( &nVertices );
+  f.WriteInt     ( &Version       );
+  f.CreateWrite  ( name           );
+  f.WriteTerLine ( devChain,False );
+  f.WriteInt     ( &nVertices     );
   for (i=0;i<nVertices;i++)
     StreamWrite ( f,Vertex[i] );
 
@@ -1067,10 +1081,10 @@ int i,Version;
 
   FreeMemory();
 
-  f.ReadInt    ( &Version   );
-  f.CreateRead ( name       );
-  f.ReadFile   ( devChain,sizeof(devChain) );
-  f.ReadInt    ( &nVertices );
+  f.ReadInt     ( &Version       );
+  f.CreateRead  ( name           );
+  f.ReadTerLine ( devChain,False );
+  f.ReadInt     ( &nVertices     );
   if (nVertices>0)  {
     nVAlloc = nVertices;
     Vertex  = new PCSSVertex[nVertices];
