@@ -1,4 +1,4 @@
-// $Id: ss_vxedge.cpp$
+// $Id: ss_vxedge.cpp,v 1.5 2006/07/12 09:39:10 mdw Exp $
 /*
      This library is free software; you can redistribute it and/or
      modify it under the terms of the GNU Lesser General Public
@@ -66,17 +66,17 @@ int      sd_min_len    = 3;       // minimal length of strands
 
 realtype hx_nres_rtol  = 0.2;     // rel tolerance for length of helices
 realtype sd_nres_rtol  = 0.2;     // rel tolerance for length of strands
-int      hx_nres_atol  = 1;       // abs tolerance for length of helices
-int      sd_nres_atol  = 1;       // abs tolerance for length of strands
+int      hx_nres_atol  = 6;       // abs tolerance for length of helices
+int      sd_nres_atol  = 3;       // abs tolerance for length of strands
 
-realtype length_rtol   = 0.01;    // rel tol-ce for edge lengths
-realtype length_atol   = 0.0;     // abs tol-ce for edge lengths
-realtype ev_uncert_min = Pi/18.0; // min unc-ty in edge-vertex angles
-realtype vv_uncert_min = Pi/9.0;  // min unc-ty in vertex-vertex angles
-realtype tn_uncert_min = Pi/9.0;  // min unc-ty in torsion angles
-realtype ev_uncert_max = Pi/2.0;  // max unc-ty in edge-vertex angles
-realtype vv_uncert_max = Pi/2.0;  // max unc-ty in vertex-vertex angles
-realtype tn_uncert_max = Pi/6.0;  // max unc-ty in torsion angles
+realtype length_rtol   = 0.2;     // rel tol-ce for edge lengths
+realtype length_atol   = 1.75;    // abs tol-ce for edge lengths
+realtype ev_uncert_min = Pi/9.0;  // min unc-ty in edge-vertex angles
+realtype vv_uncert_min = Pi/12.0; // min unc-ty in vertex-vertex angles
+realtype tn_uncert_min = Pi/12.0; // min unc-ty in torsion angles
+realtype ev_uncert_max = Pi/6.0;  // max unc-ty in edge-vertex angles
+realtype vv_uncert_max = Pi/8.0;  // max unc-ty in vertex-vertex angles
+realtype tn_uncert_max = Pi/9.0;  // max unc-ty in torsion angles
 
 int CheckSSConnectivity = CSSC_None;
 
@@ -315,7 +315,7 @@ void CSSVertex::GetPosition ( realtype & vx0, realtype & vy0,
 }
 
 int CSSVertex::GetPositions ( PCMMDBManager MMDB, int minlen )  {
-int        i,selHnd_ca;
+int        selHnd_ca;
 PPCAtom    CA;
 
   initPos = MMDB->GetResidueNo ( 1,chainID,initSeqNum,initICode );
@@ -597,10 +597,10 @@ int dn;
   switch (type)  {
     case V_HELIX  : if (v->classID!=classID)  return -2.0;
                     dn = nres + v->nres;
-                    if (dn>0)  return 2.0*fabs(v->nres-nres)/dn;
+                    if (dn>0)  return 2.0*fabs(realtype(v->nres-nres))/dn;
                          else  return 0.0;
     case V_STRAND : dn = nres + v->nres;
-                    if (dn>0)  return 2.0*fabs(v->nres-nres)/dn;
+                    if (dn>0)  return 2.0*fabs(realtype(v->nres-nres))/dn;
                          else  return 0.0;
     default : ;  
   }
@@ -673,7 +673,6 @@ void  CompareStr ( pstr s1, pstr s2, pstr name )  {
 }
 
 void  CSSVertex::Copy ( PCSSVertex v )  {
-int i;
 
   FreeMemory();
 
@@ -714,43 +713,43 @@ int i;
 void  CSSVertex::write ( RCFile f )  {
 int Version=3;
 
-  f.WriteInt    ( &Version  );
+  f.WriteInt     ( &Version  );
 
-  f.WriteInt    ( &id       );
-  f.WriteInt    ( &type     );
-  f.WriteInt    ( &classID  );
-  f.WriteInt    ( &nres     );
-  f.WriteReal   ( &x0       );
-  f.WriteReal   ( &y0       );
-  f.WriteReal   ( &z0       );
-  f.WriteReal   ( &ex       );
-  f.WriteReal   ( &ey       );
-  f.WriteReal   ( &ez       );
-  f.WriteReal   ( &dalpha   );
-  f.WriteReal   ( &length   );
-  f.WriteReal   ( &mass     );
+  f.WriteInt     ( &id       );
+  f.WriteInt     ( &type     );
+  f.WriteInt     ( &classID  );
+  f.WriteInt     ( &nres     );
+  f.WriteFloat   ( &x0       );
+  f.WriteFloat   ( &y0       );
+  f.WriteFloat   ( &z0       );
+  f.WriteFloat   ( &ex       );
+  f.WriteFloat   ( &ey       );
+  f.WriteFloat   ( &ez       );
+  f.WriteFloat   ( &dalpha   );
+  f.WriteFloat   ( &length   );
+  f.WriteFloat   ( &mass     );
 
-  f.CreateWrite ( name      );
-  f.WriteInt    ( &serNum   );
-  f.WriteInt    ( &strandNo );
-  f.WriteFile   ( vertexID   ,sizeof(vertexID)    );
-  f.WriteFile   ( chainID    ,sizeof(chainID)     );
-  f.WriteFile   ( initResName,sizeof(initResName) );
-  f.WriteFile   ( initICode  ,sizeof(initICode)   );
-  f.WriteFile   ( endResName ,sizeof(endResName)  );
-  f.WriteFile   ( endICode   ,sizeof(endICode)    );
-  f.WriteInt    ( &initSeqNum );
-  f.WriteInt    ( &endSeqNum  );
-  f.WriteInt    ( &initPos    );
-  f.WriteInt    ( &endPos     );
-  f.WriteInt    ( &VNo        );
+  f.CreateWrite  ( name      );
+  f.WriteInt     ( &serNum   );
+  f.WriteInt     ( &strandNo );
+  f.WriteTerLine ( vertexID   ,False );
+  f.WriteTerLine ( chainID    ,False );
+  f.WriteTerLine ( initResName,False );
+  f.WriteTerLine ( initICode  ,False );
+  f.WriteTerLine ( endResName ,False );
+  f.WriteTerLine ( endICode   ,False );
+  f.WriteInt     ( &initSeqNum );
+  f.WriteInt     ( &endSeqNum  );
+  f.WriteInt     ( &initPos    );
+  f.WriteInt     ( &endPos     );
+  f.WriteInt     ( &VNo        );
 
-  f.WriteReal   ( &x1         );
-  f.WriteReal   ( &x2         );
-  f.WriteReal   ( &y1         );
-  f.WriteReal   ( &y2         );
-  f.WriteReal   ( &z1         );
-  f.WriteReal   ( &z2         );
+  f.WriteFloat   ( &x1         );
+  f.WriteFloat   ( &x2         );
+  f.WriteFloat   ( &y1         );
+  f.WriteFloat   ( &y2         );
+  f.WriteFloat   ( &z1         );
+  f.WriteFloat   ( &z2         );
 
 }
 
@@ -760,43 +759,43 @@ int Version;
 
   FreeMemory();
 
-  f.ReadInt    ( &Version  );
+  f.ReadInt     ( &Version  );
 
-  f.ReadInt    ( &id       );
-  f.ReadInt    ( &type     );
-  f.ReadInt    ( &classID  );
-  f.ReadInt    ( &nres     );
-  f.ReadReal   ( &x0       );
-  f.ReadReal   ( &y0       );
-  f.ReadReal   ( &z0       );
-  f.ReadReal   ( &ex       );
-  f.ReadReal   ( &ey       );
-  f.ReadReal   ( &ez       );
-  f.ReadReal   ( &dalpha   );
-  f.ReadReal   ( &length   );
-  f.ReadReal   ( &mass     );
+  f.ReadInt     ( &id       );
+  f.ReadInt     ( &type     );
+  f.ReadInt     ( &classID  );
+  f.ReadInt     ( &nres     );
+  f.ReadFloat   ( &x0       );
+  f.ReadFloat   ( &y0       );
+  f.ReadFloat   ( &z0       );
+  f.ReadFloat   ( &ex       );
+  f.ReadFloat   ( &ey       );
+  f.ReadFloat   ( &ez       );
+  f.ReadFloat   ( &dalpha   );
+  f.ReadFloat   ( &length   );
+  f.ReadFloat   ( &mass     );
 
-  f.CreateRead ( name      );
-  f.ReadInt    ( &serNum   );
-  f.ReadInt    ( &strandNo );
-  f.ReadFile   ( vertexID   ,sizeof(vertexID)    );
-  f.ReadFile   ( chainID    ,sizeof(chainID)     );
-  f.ReadFile   ( initResName,sizeof(initResName) );
-  f.ReadFile   ( initICode  ,sizeof(initICode)   );
-  f.ReadFile   ( endResName ,sizeof(endResName)  );
-  f.ReadFile   ( endICode   ,sizeof(endICode)    );
-  f.ReadInt    ( &initSeqNum );
-  f.ReadInt    ( &endSeqNum  );
-  f.ReadInt    ( &initPos    );
-  f.ReadInt    ( &endPos     );
-  f.ReadInt    ( &VNo        );
+  f.CreateRead  ( name      );
+  f.ReadInt     ( &serNum   );
+  f.ReadInt     ( &strandNo );
+  f.ReadTerLine ( vertexID   ,False );
+  f.ReadTerLine ( chainID    ,False );
+  f.ReadTerLine ( initResName,False );
+  f.ReadTerLine ( initICode  ,False );
+  f.ReadTerLine ( endResName ,False );
+  f.ReadTerLine ( endICode   ,False );
+  f.ReadInt     ( &initSeqNum );
+  f.ReadInt     ( &endSeqNum  );
+  f.ReadInt     ( &initPos    );
+  f.ReadInt     ( &endPos     );
+  f.ReadInt     ( &VNo        );
 
-  f.ReadReal   ( &x1         );
-  f.ReadReal   ( &x2         );
-  f.ReadReal   ( &y1         );
-  f.ReadReal   ( &y2         );
-  f.ReadReal   ( &z1         );
-  f.ReadReal   ( &z2         );
+  f.ReadFloat   ( &x1         );
+  f.ReadFloat   ( &x2         );
+  f.ReadFloat   ( &y1         );
+  f.ReadFloat   ( &y2         );
+  f.ReadFloat   ( &z1         );
+  f.ReadFloat   ( &z2         );
 
 }
 
@@ -1004,7 +1003,7 @@ int      i,j,na;
   //  from which it follows that
   //      Torsion(U,W,V) = Torsion(V,-W,U)
 
-  alpha4  = GetTorsion(U,W,V);
+  alpha4  = GetTorsion((rvector)U,(rvector)W,(rvector)V);
   dalpha4 = RMin(RMax(v1->dalpha+v2->dalpha,tn_uncert_min),tn_uncert_max);
 
   GoodTorsion = (alpha4>NO_TORSION)       &&  // the torsion is defined
@@ -1116,48 +1115,48 @@ int bdir1,bdir2;
 
 void  CSSEdge::write ( RCFile f )  {
 int Version=1;
-  f.WriteInt  ( &Version     );
-  f.WriteInt  ( &id1         );
-  f.WriteInt  ( &id2         );
-  f.WriteInt  ( &bdir        );
-  f.WriteInt  ( &vtype1      );
-  f.WriteInt  ( &vtype2      );
-  f.WriteReal ( &length      );
-  f.WriteReal ( &ex          );
-  f.WriteReal ( &ey          );
-  f.WriteReal ( &ez          );
-  f.WriteReal ( &alpha1      );
-  f.WriteReal ( &alpha2      );
-  f.WriteReal ( &alpha3      );
-  f.WriteReal ( &alpha4      );
-  f.WriteReal ( &dalpha1     );
-  f.WriteReal ( &dalpha2     );
-  f.WriteReal ( &dalpha3     );
-  f.WriteReal ( &dalpha4     );
-  f.WriteBool ( &GoodTorsion );
+  f.WriteInt   ( &Version     );
+  f.WriteInt   ( &id1         );
+  f.WriteInt   ( &id2         );
+  f.WriteInt   ( &bdir        );
+  f.WriteInt   ( &vtype1      );
+  f.WriteInt   ( &vtype2      );
+  f.WriteFloat ( &length      );
+  f.WriteFloat ( &ex          );
+  f.WriteFloat ( &ey          );
+  f.WriteFloat ( &ez          );
+  f.WriteFloat ( &alpha1      );
+  f.WriteFloat ( &alpha2      );
+  f.WriteFloat ( &alpha3      );
+  f.WriteFloat ( &alpha4      );
+  f.WriteFloat ( &dalpha1     );
+  f.WriteFloat ( &dalpha2     );
+  f.WriteFloat ( &dalpha3     );
+  f.WriteFloat ( &dalpha4     );
+  f.WriteBool  ( &GoodTorsion );
 }
 
 void  CSSEdge::read ( RCFile f )  {
 int Version;
-  f.ReadInt  ( &Version     );
-  f.ReadInt  ( &id1         );
-  f.ReadInt  ( &id2         );
-  f.ReadInt  ( &bdir        );
-  f.ReadInt  ( &vtype1      );
-  f.ReadInt  ( &vtype2      );
-  f.ReadReal ( &length      );
-  f.ReadReal ( &ex          );
-  f.ReadReal ( &ey          );
-  f.ReadReal ( &ez          );
-  f.ReadReal ( &alpha1      );
-  f.ReadReal ( &alpha2      );
-  f.ReadReal ( &alpha3      );
-  f.ReadReal ( &alpha4      );
-  f.ReadReal ( &dalpha1     );
-  f.ReadReal ( &dalpha2     );
-  f.ReadReal ( &dalpha3     );
-  f.ReadReal ( &dalpha4     );
-  f.ReadBool ( &GoodTorsion );
+  f.ReadInt   ( &Version     );
+  f.ReadInt   ( &id1         );
+  f.ReadInt   ( &id2         );
+  f.ReadInt   ( &bdir        );
+  f.ReadInt   ( &vtype1      );
+  f.ReadInt   ( &vtype2      );
+  f.ReadFloat ( &length      );
+  f.ReadFloat ( &ex          );
+  f.ReadFloat ( &ey          );
+  f.ReadFloat ( &ez          );
+  f.ReadFloat ( &alpha1      );
+  f.ReadFloat ( &alpha2      );
+  f.ReadFloat ( &alpha3      );
+  f.ReadFloat ( &alpha4      );
+  f.ReadFloat ( &dalpha1     );
+  f.ReadFloat ( &dalpha2     );
+  f.ReadFloat ( &dalpha3     );
+  f.ReadFloat ( &dalpha4     );
+  f.ReadBool  ( &GoodTorsion );
 }
 
 
