@@ -1,3 +1,4 @@
+// $Id: ssm_align.cpp,v 1.4 2007/07/06 15:37:49 mdw Exp $
 /*
      This library is free software; you can redistribute it and/or
      modify it under the terms of the GNU Lesser General Public
@@ -17,9 +18,9 @@
 */
 // =================================================================
 //
-//    22.04.04   <--  Date of Last Modification.
+//    26.06.06   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-//  -------------------------------------------------------------------
+//  ----------------------------------------------------------------
 //
 //  **** Module  :  SSM_Align <interface>
 //       ~~~~~~~~~
@@ -29,7 +30,7 @@
 //       ~~~~~~~~~  CXAlign   ( Output alignment             ) 
 //                  CXTAlign  ( Text output alignment        )
 //
-//  E. Krissinel, 2002-2004
+//  E. Krissinel, 2002-2006
 //
 // =================================================================
 //
@@ -92,27 +93,32 @@ void CSSMAlign::InitSSMAlign()  {
   seqIdentity = 0;    // sequence identity
   ncombs      = 1.0;  // number of SSE combinations
 
-  selHndCa1   = 0;    // selection handle to used C-alphas in query structure
-  Ca1         = NULL; // C-alpha correspondence vector for query structure
-  dist1       = NULL; // opt-d distances between the query and target C-alphas
-  selHndCa2   = 0;    // selection handle to used C-alphas in target structure
-  Ca2         = NULL; // C-alpha correspondence vector for target structure
+  selHndCa1   = 0;    // selection handle to used C-alphas in
+                      //   query structure
+  Ca1         = NULL; // C-alpha correspondence vector for
+                      //   query structure
+  dist1       = NULL; // optimized distances between the query
+                      //   and target C-alphas
+  selHndCa2   = 0;    // selection handle to used C-alphas in
+                      //   target structure
+  Ca2         = NULL; // C-alpha correspondence vector for
+                      //   target structure
 
   G1          = NULL;
-  G2          = NULL;       // retained SSE graphs
+  G2          = NULL; // retained SSE graphs
 
 }
 
 
 void CSSMAlign::MapSelections ( int & selHndCa, PCMMDBManager M,
-                                 PCSSGraph G, int selHnd,
-                                 ivector & newID )  {
+                                PCSSGraph G, int selHnd,
+                                ivector & newID )  {
 PPCAtom a;
 int     nr,i,k;
-  G->SelectCalphas ( M,selHndCa,"*" );
   if (selHnd>0)  {
-    M->GetSelIndex  ( selHndCa,a,nr );
-    GetVectorMemory ( newID,nr,0 );
+    G->SelectCalphas ( M,selHndCa,NULL );
+    M->GetSelIndex   ( selHndCa,a,nr   );
+    GetVectorMemory  ( newID,nr,0      );
     k = 0;
     for (i=0;i<nr;i++)
       if (a[i]->isInSelection(selHnd)) newID[i] = k++;
@@ -130,29 +136,39 @@ int     i,k;
 
   MapSelections ( selHndCa1,M1,G1,selHnd1,newID1 );
   MapSelections ( selHndCa2,M2,G2,selHnd2,newID2 );
-  
-  if (newID2)
-    for (i=0;i<nres1;i++)
-      if (Ca1[i]>=0)  Ca1[i] = newID2[Ca1[i]];
-
-  if (newID1)
-    for (i=0;i<nres2;i++)
-      if (Ca2[i]>=0)  Ca2[i] = newID1[Ca2[i]];
 
   if (newID2)  {
     k = 0;
-    for (i=0;i<nres2;i++)
-      if (newID2[i]>=0)  Ca2[k++] = Ca2[i];
+    if (newID1)  {
+      for (i=0;i<nres2;i++)
+        if (newID2[i]>=0)  {
+          if (Ca2[i]>=0)  Ca2[k++] = newID1[Ca2[i]];
+                    else  Ca2[k++] = -1;
+        }
+    } else  {
+      for (i=0;i<nres2;i++)
+        if (newID2[i]>=0)  Ca2[k++] = Ca2[i];
+    }
   }
 
   if (newID1)  {
     k = 0;
-    for (i=0;i<nres1;i++)
-      if (newID1[i]>=0)  {
-        Ca1[k]   = Ca1[i];
-        dist1[k] = dist1[i];
-        k++;
-      }
+    if (newID2)  {
+      for (i=0;i<nres1;i++)
+        if (newID1[i]>=0)  {
+          if (Ca1[i]>=0)  Ca1[k] = newID2[Ca1[i]];
+                    else  Ca1[k] = -1;
+          dist1[k] = dist1[i];
+          k++;
+        }
+    } else  {
+      for (i=0;i<nres1;i++)
+        if (newID1[i]>=0)  {
+          Ca1[k]   = Ca1[i];
+          dist1[k] = dist1[i];
+          k++;
+        }
+    }
   }
 
   nsel1 = M1->GetSelLength ( selHndCa1 );
@@ -164,7 +180,8 @@ int     i,k;
 }
 
 
-PCSSGraph CSSMAlign::GetSSGraph ( PCMMDBManager M, int selHnd, int & rc )  {
+PCSSGraph CSSMAlign::GetSSGraph ( PCMMDBManager M, int selHnd,
+                                  int & rc )  {
 PCSSGraph G;
 
   G  = new CSSGraph();
@@ -221,7 +238,8 @@ int        i,nMatches,nm;
   for (i=0;i<nMatches;i++)
     if (Match[i])  {
       Match[i]->GetMatch ( F1,F2,nm );
-      Superpose.SuperposeCalphas ( G1,G2,F1,F2,nm,M1,M2,selHnd1,selHnd2 );
+      Superpose.SuperposeCalphas ( G1,G2,F1,F2,nm,M1,M2,
+                                   selHnd1,selHnd2 );
       Q1 = Superpose.GetCalphaQ();
       if ((Q1>0.0) && (Q1>Q))  {
         Q = Q1;
@@ -347,9 +365,10 @@ void CXAlign::FreeMemory()  {
 
 void CXAlign::customInit() {}
 
-void CXAlign::XAlign ( PCSSGraph g1, PPCAtom Calpha1, ivector Ca1, int nat1,
-                       PCSSGraph g2, PPCAtom Calpha2, ivector Ca2, int nat2,
-                       rvector dist1, int & nr )  {
+void CXAlign::XAlign (
+                PCSSGraph g1, PPCAtom Calpha1, ivector Ca1, int nat1,
+                PCSSGraph g2, PPCAtom Calpha2, ivector Ca2, int nat2,
+               rvector dist1, int & nr )  {
 int i,j;
 
   FreeMemory();
@@ -646,7 +665,8 @@ char sse[2],hp[2],ch[3],S[200];
   }
   ch[2] = char(0);
 
-  sprintf ( S,"|%1s%1s %2s%3s%4i%1s|",sse,hp,ch,resName,seqNum,insCode );
+  sprintf ( S,"|%1s%1s %2s%3s%4i%1s|",
+              sse,hp,ch,resName,seqNum,insCode );
   f.Write ( S );
 
 }
@@ -658,7 +678,8 @@ int  i;
   if (alignKey<4)  {
 
     if (alignKey!=2)
-      PrintAtom ( f,sseType1,hydropathy1,chID1,resName1,seqNum1,insCode1 );
+      PrintAtom ( f,sseType1,hydropathy1,chID1,
+                    resName1,seqNum1,insCode1 );
     else
       f.Write ( "|             |" );
 
@@ -686,7 +707,8 @@ int  i;
     }
 
     if (alignKey!=3)
-      PrintAtom ( f,sseType2,hydropathy2,chID2,resName2,seqNum2,insCode2 );
+      PrintAtom ( f,sseType2,hydropathy2,chID2,
+                    resName2,seqNum2,insCode2 );
     else
       f.Write ( "|             |" );
 
@@ -851,6 +873,14 @@ int         nat1,nat2,nr,j;
   f.WriteLine ( " ::    similarity 1" );
   f.WriteLine ( " ..    dissimilar residues: similarity 0" );
 
+  f.WriteLine ( " " );
+  f.WriteLine ( " Sequence alignment from SSM:" );
+  f.WriteLine ( " " );
+  pstr algn1=NULL, algn2=NULL;
+  CXA.GetAlignments ( algn1, algn2 );
+  f.WriteLine ( algn1 );
+  f.WriteLine ( algn2 );
+
 }
 
 
