@@ -1,4 +1,4 @@
-// $Id: ssm_superpose.cpp$
+// $Id: ssm_superpose.cpp,v 1.3 2006/07/12 09:39:10 mdw Exp $
 /*
      This library is free software; you can redistribute it and/or
      modify it under the terms of the GNU Lesser General Public
@@ -18,16 +18,16 @@
 */
 // =================================================================
 //
-//    30.04.04   <--  Date of Last Modification.
+//    26.06.06   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 // -----------------------------------------------------------------
 //
 //  **** Module  :  ssm_superpose  <interface>
 //       ~~~~~~~~~
-//  **** Functions : SuperposeCalphas ( superposing protein structures )
+//  **** Functions : SuperposeCalphas (superposing protein structures)
 //       ~~~~~~~~~~~
 //
-//  (C) E. Krissinel 2002-2004
+//  (C) E. Krissinel 2002-2006
 //
 // =================================================================
 //
@@ -42,7 +42,7 @@
 
 
 #ifndef  __BFGS_Min__
-#include "mmdb/bfgs_min.h"
+#include <mmdb/bfgs_min.h>
 #endif
 
 #ifndef  __SSM_Superpose__
@@ -355,7 +355,8 @@ void CSuperpose::SetIterationLimits ( int iter_max, int iter_min,
   maxHollowIt = max_hollow;
 }
 
-void  CSuperpose::SetCaSelections ( pstr selection1, pstr selection2 )  {
+void  CSuperpose::SetCaSelections ( pstr selection1,
+                                    pstr selection2 )  {
   CreateCopy ( selString1,selection1 );
   CreateCopy ( selString2,selection2 );
 }
@@ -383,13 +384,17 @@ int  CSuperpose::GetNAlign()  {
 
 
 void  CSuperpose::GetSuperposition ( ivector  & Ca1  ,
-                                     rvector  & dist1, int & nCa1,
-                                     ivector  & Ca2  , int & nCa2,
+                                     rvector  & dist1,
+                                     int      & nCa1 ,
+                                     ivector  & Ca2  ,
+                                     int      & nCa2 ,
                                      mat44    & TMat ,
                                      realtype & rmsdAchieved,
-                                     int & nAligned,   int & nGaps,
-                                     realtype & seqIdentity,
-                                     int & nMisD, realtype & nCombs )  {
+                                     int      & nAligned    ,
+                                     int      & nGaps       ,
+                                     realtype & seqIdentity ,
+                                     int      & nMisD       ,
+                                     realtype & nCombs )  {
 //
 //   Ca1[i]>=0 gives the index of Calpha of 2nd structure,
 // superposed on ith Calpha of 1st structure (0<=i<nres1).
@@ -417,7 +422,8 @@ int i,j;
       if (Ca1[i]>=0)  dist1[i] = sqrt(a1[i].dist0);
                 else  dist1[i] = -1.0;
     }
-    for (i=0;i<nres2;i++)  Ca2[i] = a2[i].c0;
+    for (i=0;i<nres2;i++)
+      Ca2[i] = a2[i].c0;
     for (i=0;i<4;i++)
       for (j=0;j<4;j++)
         TMat[i][j] = TMatrix[i][j];
@@ -741,7 +747,7 @@ Boolean    AddEdges1,AddEdges2;
 
   BFGS->SetMinFunction ( &SMTF,MinTFunc );
 
-  BFGS->BFGS_Driver ( 6,T,TypT,B,i,    // MinN,x0,TypX,FuncValue,TermCode
+  BFGS->BFGS_Driver ( 6,T,TypT,B,i,  // MinN,x0,TypX,FuncValue,TermCode
                       0,100,1.0,       // Digits,ItnLmt,TypF
                       0.0,0.0,MaxReal, // GrdTol,StpTol,MaxStp
                       False,NULL,NULL  // Hess,LowLimit,TopLimit
@@ -821,7 +827,8 @@ int i;
 
 
 void CSuperpose::MapSSEs ( PPCAtom Calpha, PSSpAtom a, int nres,
-                           PCSSGraph G, RPSSSEDesc SSED, int & nSSEs )  {
+                           PCSSGraph G, RPSSSEDesc SSED,
+                           int & nSSEs )  {
 //  Returns vector SSED.pos containing the origin of SSE sequences in
 //  array a, and vector SSED.len containing the SSEs' lengths.
 //  nSSEs returns the number of SSEs.
@@ -845,7 +852,8 @@ int        i,j,vtype,k, p1,p2, c1,c2;
     SSED = new SSSEDesc[nSSEs];
   
     for (i=1;i<=nSSEs;i++)  {
-      G->GetVertexRange ( i,chID,initSeqNum,initICode,endSeqNum,endICode );
+      G->GetVertexRange ( i,chID,initSeqNum,initICode,
+                                 endSeqNum,endICode );
       vtype = G->GetVertexType(i);
       p1 = -1;
       p2 = -1;
@@ -857,10 +865,10 @@ int        i,j,vtype,k, p1,p2, c1,c2;
           c2 = j;
           seqNum = Calpha[j]->GetSeqNum ();
           iCode  = Calpha[j]->GetInsCode();
-          if ((p1<0) && (initSeqNum==seqNum) && (!strcmp(initICode,iCode)))
-              p1 = j;
-          if ((p2<0) && (endSeqNum==seqNum) && (!strcmp(endICode,iCode)))
-              p2 = j;
+          if ((p1<0) && (initSeqNum==seqNum) &&
+              (!strcmp(initICode,iCode)))    p1 = j;
+          if ((p2<0) && (endSeqNum==seqNum) && 
+              (!strcmp(endICode,iCode)))     p2 = j;
         }
       }
       k = i-1;
@@ -968,7 +976,7 @@ void  CSuperpose::CalcDistance ( int SSE1, int SSE2,
 //  other parameters needed for 3D alignment of the SSEs.
 realtype d,d1;
 int      minAlign, pos1,pos2, len1,len2, i,j,k,l, i1,i2;
-int      p1,p2,e1,e2;
+ int      p1, p2;
 
 
   //  1. Initial preparations
@@ -1086,8 +1094,8 @@ void  CSuperpose::AlignSSEs ( RSSectionDist D, int unmap )  {
 //  and Calpha2 from pos1 to pos1+len1-1 and pos2 to pos2+len2-1,
 //  respectively using data obtained from calcDistance.
 //    The correspondence is returned as pairs  c2[c1[i]]=i and
-//  c1[c2[j]]=j for atoms i and j; c1[k] and c2[m] for non-corresponding
-//  atoms k and m are not changed.
+//  c1[c2[j]]=j for atoms i and j; c1[k] and c2[m] for
+//  non-corresponding atoms k and m are not changed.
 int  i1,i2, c1;
 
   if (D.na>0)  {
@@ -1155,7 +1163,6 @@ Boolean  CSuperpose::isMC ( int pos1, int pos2 )  {
 int i;
 
   if (allowMC)  return False;
-//  if ((pos1<0) || (pos2<0)) return False;  //  to be removed 30.03.2003
 
   i = pos1 + 1;
   while (i<nres1)
@@ -1203,7 +1210,8 @@ int      i,j, k1,k2, i1,j1;
           (SSED1[k1].classID==SSED2[k2].classID) &&
           (!isMC(SSED1[k1].pos,SSED2[k2].pos)))  {
         CalcDistance ( F1[i],F2[j],SDist[i1][j1] );
-        if ((SDist[i1][j1].na<=0) || (SDist[i1][j1].cosine<minCosine) ||
+        if ((SDist[i1][j1].na<=0) || 
+            (SDist[i1][j1].cosine<minCosine) ||
             (SDist[i1][j1].rmsd>rmsd2))
           SDist[i1][j1].rmsd = -1.0;
       } else
@@ -1306,7 +1314,7 @@ PSContact contact;
 ivector   ip,im;
 vect3     v1,v2;
 realtype  l1,l2,cosine;
-int       ncontacts,i,j,k,m1,m2,n1,n2,i1,i2;
+int       ncontacts,i,j,m1,m2,n1,n2,i1,i2;
 
   //  1. Find all contacts in the range og 0.0 - Rmsd
   contact   = NULL;
@@ -1315,13 +1323,15 @@ int       ncontacts,i,j,k,m1,m2,n1,n2,i1,i2;
                      contact,ncontacts,0,NULL,0,
                      BRICK_ON_2 | BRICK_READY );
 
-  //  2. Leave only open contacts (i.e. those between yet unmapped atoms),
-  //     with regard to the SSE type
+  //  2. Leave only open contacts (i.e. those between yet unmapped
+  //     atoms), with regard to the SSE type
   j = 0;
   for (i=0;i<ncontacts;i++)  {
     i1 = contact[i].id1;
     i2 = contact[i].id2;
-    if ((a1[i1].c<0) && (a2[i2].c<0) && a1[i1].CompatibleSSE(a2[i2]))  {
+    if ((!a1[i1].excluded) && (a1[i1].c<0) &&
+        (!a2[i2].excluded) && (a2[i2].c<0) &&
+        a1[i1].CompatibleSSE(a2[i2])) {
       if (j<i)  contact[j].Copy ( contact[i] );
       contact[j].dist = contact[j].dist*contact[j].dist;
       j++;
@@ -1375,9 +1385,9 @@ int       ncontacts,i,j,k,m1,m2,n1,n2,i1,i2;
             cosine = (v1[0]*v2[0]+v1[1]*v2[1]+v1[2]*v2[2])/sqrt(l1*l2);
             if (cosine>=minCosine)  {
               //  The contact is valid. Because the contacts are sorted
-              // by decreasing the contact distance, we simply accept all
-              // firstly encountered contacts and by this invalidate all
-              // other contacts of the same atoms
+              // by decreasing the contact distance, we simply accept
+              // all firstly encountered contacts and by this
+              // invalidate all other contacts of the same atoms
               if (j<i)  contact[j].Copy ( contact[i] );
               // close contact
               a1[i1].c    = i2;
@@ -1423,6 +1433,8 @@ void  CSuperpose::RecoverGaps ( PPCAtom Ca1, PSSpAtom at1, int nat1,
 realtype thr2,d,d2;
 int      i, k1,k2,n1,n2,sw;
 Boolean  B1,B2;
+ 
+ d = 0.0;
 
   thr2 = thresh*thresh;
   i    = 0;
@@ -1436,8 +1448,8 @@ Boolean  B1,B2;
     if (i<nat1)  {
 
       k1 = i;   // begining of a gap in 1st structure
-      if (i>0)  n1 = at1[i-1].c+1; // begining of the gap in 2nd structure
-          else  n1 = -1;
+      if (i>0)  n1 = at1[i-1].c+1; // begining of the gap in
+          else  n1 = -1;           //   2nd structure
 
       // skip all open contacts
       while (i<nat1)
@@ -1485,8 +1497,8 @@ Boolean  B1,B2;
             break;
 
       } else if ((n1>=0) && (n2>=0))  {
-        //  The gap is surrounded by closed contacts in the 1st structure.
-        //  Recover it gradually from begining and the end.
+        //  The gap is surrounded by closed contacts in the 1st
+        // structure. Recover it gradually from begining and the end.
         B1 = True;
         B2 = True;
         while ((k1<=k2) && (n1<=n2) && (B1 || B2))  {
@@ -1549,7 +1561,7 @@ void  CSuperpose::CleanShortSections ( PSSpAtom at1, int nat1,
 //  follow the same direction (along protein section) and are
 //  long enough to be significant. If sections are too short
 //  then they are unmapped.
-int  i,j,j2,k,shortSect,nmisdr1;
+  int  i,j,j2,k,shortSect;
 
   if (nmisdr<2)  shortSect = shortSect1;
            else  shortSect = shortSect2;
@@ -1639,8 +1651,8 @@ int  i,k,m;
 
 }
 
-realtype CSuperpose::CalcNCombs ( PCSSGraph G, PSSSEDesc SSED, int nSSEs,
-                                  PSSpAtom  a, int nres )  {
+realtype CSuperpose::CalcNCombs ( PCSSGraph G, PSSSEDesc SSED,
+                                  int nSSEs, PSSpAtom  a, int nres ) {
 ivector  F;
 realtype nc;
 int      i,j,k,n;
@@ -1716,7 +1728,7 @@ void CSortDist::Sort ( PSSortDistData sdata, int len )  {
 void CSuperpose::GetSSESpseCenters ( RSSSEDesc Q1, RSSSEDesc Q2,
                                      RSSSEDesc T1, RSSSEDesc T2,
                                      realtype & qc1, realtype & qc2,
-                                     realtype & tc1, realtype & tc2 )  {
+                                     realtype & tc1, realtype & tc2 ) {
 realtype d2, dq,dt, quality,quality0;
 int      iq1,pq1,eq1,lq1, it1,pt1,et1, l1;
 int      iq2,pq2,eq2,lq2, it2,pt2,et2, l2;
@@ -2063,7 +2075,8 @@ realtype  NormN,Na2,NormR;
 }
 
 
-void CSuperpose::UnmapExcluded ( PSSpAtom a1, PSSpAtom a2, int nres1 )  {
+void CSuperpose::UnmapExcluded ( PSSpAtom a1, PSSpAtom a2,
+                                 int nres1 )  {
 int i;
   for (i=0;i<nres1;i++)
     if (a1[i].excluded && (a1[i].c>=0))  {
@@ -2072,6 +2085,7 @@ int i;
     }
 }
 
+
 int  CSuperpose::OptimizeNalign()  {
 //  Finds maximal Nalign such that RMSD does not exceed the given Rmsd
 SSectionDist D;
@@ -2413,8 +2427,10 @@ int  selHnd1,selHnd2;
   //  allocates a1,2
   selHnd1 = 0;
   selHnd2 = 0;
-  SelectCalphas ( MMDB1,G1,Calpha1,a1,nres1,selHnd1,selInclHnd1,selString1 );
-  SelectCalphas ( MMDB2,G2,Calpha2,a2,nres2,selHnd2,selInclHnd2,selString2 );
+  SelectCalphas ( MMDB1,G1,Calpha1,a1,nres1,selHnd1,
+                        selInclHnd1,selString1 );
+  SelectCalphas ( MMDB2,G2,Calpha2,a2,nres2,selHnd2,
+                        selInclHnd2,selString2 );
   if ((nres1<=0) || (nres2<=0))  {
     if (nres1<=0)  rc = SPOSE_NoCalphas1;
              else  rc = SPOSE_NoCalphas2;
@@ -2429,7 +2445,6 @@ int  selHnd1,selHnd2;
     return rc;
   }
 
-
   //  2. Map vertices on the selected Calphas
 
   MapSSEs ( Calpha1,a1,nres1,G1,SSED1,nSSEs1 );
@@ -2498,11 +2513,11 @@ void  SSuperposeData::SelectCalphas()  {
 //  ----------------------------------------------------------------
 
 int  CSuperpose::SuperposeCalphas (
-            PSSuperposeData SD1,  // superposition data of 1st structure
-            PSSuperposeData SD2,  // superposition data of 2nd structure
-            ivector          F1,  // matched vertices of SD1.G [1..mlen]
-            ivector          F2,  // matched vertices of SD2.G [1..mlen]
-            int            mlen   // length of match (F1,F2)
+           PSSuperposeData SD1,  // superposition data of 1st structure
+           PSSuperposeData SD2,  // superposition data of 2nd structure
+           ivector          F1,  // matched vertices of SD1.G [1..mlen]
+           ivector          F2,  // matched vertices of SD2.G [1..mlen]
+           int            mlen   // length of match (F1,F2)
                                   )  {
 int  i,j, rc;
 
@@ -2596,7 +2611,8 @@ int  i,j, rc;
 }
 
 
-void  CSuperpose::_superpose ( PCSSGraph G1, PCSSGraph G2, int & rc )  {
+void  CSuperpose::_superpose ( PCSSGraph G1, PCSSGraph G2,
+                               int & rc )  {
 mat44 TMx0;
 int   i,j,i1,i2,AD_alloc;
 
